import pymysql
from error import ClientError
from sql_db_base import SqlBase


class MysqlClass(SqlBase):
    """
    mysql工具类(no delete)---pymysql: 1.0.2 +

    tips: {'host','port','user','password','database'}
    """

    @property
    def db(self):
        """
        连接mysql

        :return: db
        """
        try:
            db = pymysql.connect(**self.settings)
        except Exception as e:
            raise ClientError('mysql client db error: ' + str(e.args))
        return db



import platform
import cx_Oracle
from error import ClientError
from sql_db_base import SqlBase


class OracleClass(SqlBase):
    """
    oracle工具类(no delete)---cx_Oracle: 8.3.0 +

    tips: {'host','port','user','password','database'}
    """

    def __init__(self, oci_path, settings):
        super(OracleClass, self).__init__(settings)
        if platform.system() == 'Windows':
            cx_Oracle.init_oracle_client(oci_path)

    @property
    def db(self):
        """
        连接mysql

        :return: db
        """
        try:
            db = cx_Oracle.connect(
                *(self.settings.user,
                  self.settings.password,
                  f'{self.settings.host}:{self.settings.port}/{self.settings.database}'))
        except Exception as e:
            raise ClientError('oracle client db error: ' + str(e))
        return db



import os
import sys
import platform


__version__ = '1.0.3'
sys.path.append(os.path.dirname(os.path.realpath(__file__)))

if platform.system() == 'Windows':
    __all__ = ['dtime', 'log', 'mongo', 'mysql', 'oracle']
else:
    __all__ = ['dtime', 'log', 'mongo', 'mysql', 'oracle', 'xugu']



from error import ParamsError
from common import dict_to_object, judge_params, retry_result


class CursorInit:
    @classmethod
    def execute(cls, *args, **kwargs):
        ...

    @classmethod
    def executemany(cls, *args, **kwargs):
        ...

    @classmethod
    def fetchall(cls, *args, **kwargs):
        ...


class DbInit:
    @classmethod
    def cursor(cls):
        return CursorInit()

    @classmethod
    def close(cls):
        ...

    @classmethod
    def commit(cls):
        ...


class SqlClientBase:
    """
    sql连接类

    tips: {'host','port','user','password','database'}
    """
    __params_template = ['host', 'port', 'user', 'password', 'database']

    def __init__(self, settings: dict):
        """
        :param settings: 数据库信息
        """
        if not judge_params(self.__params_template, settings):
            raise ParamsError('params error')
        self.settings = dict_to_object(settings)

    @property
    def db(self):
        return DbInit()

    def release(self):
        """
        释放数据量连接
        """
        return self.db.close()


class SqlBase(SqlClientBase):
    """
    sql工具类(no delete)
    """

    @retry_result
    def search(self, sql: str):
        """
        mysql查询

        :param sql: sql语句
        :return: data
        """
        cur = self.db.cursor()
        cur.execute(sql)
        return cur.fetchall()

    @retry_result
    def insert(self, sql: str):
        """
        mysql单条数据插入

        :param sql: sql语句
        :return: data
        """
        cur = self.db.cursor()
        cur.execute(sql)
        self.db.commit()
        return True

    @retry_result
    def insert_many(self, sql: str, data: list):
        """
        mysql多条数据插入

        :param sql: sql语句
        :param data: 数据集合，List[Tuple]
        :return: data
        """
        cur = self.db.cursor()
        cur.executemany(sql, data)
        self.db.commit()
        return True

    @retry_result
    def update(self, sql: str):
        """
        mysql单条数据更新

        :param sql: sql语句
        :return: data
        """
        cur = self.db.cursor()
        cur.execute(sql)
        self.db.commit()
        return True

    @retry_result
    def update_many(self, sql: str, data: list):
        """
        多约束多条数据更新

        :param sql: sql语句
        :param data: 数据集合,包含约束数据、更新数据，List[Tuple]
        :return: data
        """
        cur = self.db.cursor()
        cur.executemany(sql, data)
        self.db.commit()
        return True



import xgcondb
import platform
from sql_db_base import SqlBase
from error import ClientError, InitDBError


class XGClass(SqlBase):
    """
    虚谷工具类(no delete)---包内自带此连接工具，限linux

    tips: {'host','port','user','password','database'}
    """
    def __init__(self, settings):
        super(XGClass, self).__init__(settings)
        if platform.system() == 'Windows':
            raise InitDBError('xu_gu db not init on windows')

    @property
    def db(self):
        """
        连接虚谷

        :return: db
        """
        try:
            db = xgcondb.connect(charset='utf8', **self.settings)
        except Exception as e:
            raise ClientError('xg client db error: ' + str(e.args))
        return db



import pymongo
from typing import List
from pymongo import MongoClient
from error import ParamsError, ClientError
from common import dict_to_object, judge_params, retry_result


class MongoClass:
    """
    mongodb工具类(no delete)---pymongo: 3.8.0 +

    tips: {'host':''/List,'port','user','password','db','is_group':True/False}
    """
    __params_template = ['host', 'port', 'user', 'password', 'db', 'is_group']

    def __init__(self, settings: dict):
        """
        :param settings: 数据库信息
        """
        if not judge_params(self.__params_template, settings):
            raise ParamsError('params error')
        self.settings = dict_to_object(settings)

    @property
    def client(self):
        """
        连接mongodb

        :return: client
        """
        try:
            if self.settings.is_group:
                client = MongoClient(self.settings.host)
            else:
                client = MongoClient(host=self.settings.host, port=self.settings.port)
            client.admin.authenticate(self.settings.user, self.settings.password)
        except Exception as e:
            raise ClientError('mongo client error: ' + str(e.args))
        return client

    @property
    def db(self):
        """
        连接数据库

        :return: db连接
        """
        try:
            client = self.client
            return client[self.settings.db]
        except Exception as e:
            raise ClientError('client db error: ' + str(e.args))

    def release(self):
        """
        释放mongo连接
        """
        return self.client.close()

    @retry_result
    def search(self, table: str, query_dict: dict):
        """
        :param table: 目标数据表
        :param query_dict: query
        :return: result
        """
        db = self.db[table]
        return db.find(query_dict)

    @retry_result
    def insert(self, table: str, query_dict: dict):
        """
        :param table: 目标数据表
        :param query_dict: query
        :return: result
        """
        db = self.db[table]
        return db.insert_one(query_dict)

    @retry_result
    def insert_many(self, table: str, query_dict: List[dict]):
        """
        :param table: 目标数据表
        :param query_dict: query
        :return: result
        """
        db = self.db[table]
        return db.insert_many(query_dict)

    @retry_result
    def update(self, table: str, filter_query: dict, set_query: dict):
        """
        更新操作，查询到约束条件则更新，否则执行插入操作

        tips: db.test.update({'_id': 'id'}, {'$set': {'b': 'b'}}, true)

        :param table: 目标数据表
        :param filter_query: 查询query
        :param set_query: 数据query
        :return: result
        """
        db = self.db[table]
        return db.update_one(filter_query, set_query, upsert=True)

    @retry_result
    def update_many(self, table: str, bulk_query: List[List[dict]]):
        """
        多约束多数据更新操作，查询到约束条件则更新，否则执行插入操作

        tips1: [[{'sex': '男'}, {'$set': {'is_qualified': False}}],...]

        tips2: db.bulk_write([
          pymongo.UpdateMany({'sex': '男'}, {'$set': {'is_qualified': False}}),

          pymongo.UpdateMany({'sex': '女'}, {'$set': {'is_qualified': True}})])

        :param table: 目标数据表
        :param bulk_query: 批量操作query集合
        :return: result
        """
        db = self.db[table]
        bulk_query = [pymongo.UpdateMany(query[0], query[1], upsert=True) for query in bulk_query]
        return db.bulk_write(bulk_query)

    # def delete(self, table: str, query_dict: dict):
    #     """
    #     :param table: 目标数据表
    #     :param query_dict: query
    #     :return: result
    #     """
    #     db = self.db[table]
    #     return db.delete_one(query_dict)
    #
    # def delete_many(self, table: str, query_dict: dict):
    #     """
    #     :param table: 目标数据表
    #     :param query_dict: query
    #     :return: result
    #     """
    #     db = self.db[table]
    #     return db.delete_many(query_dict)



from dateutil.relativedelta import relativedelta
from error import TargetError, ParamsError
from datetime import datetime, timedelta
from typing import Union

# days_of_month_non_leap_year = [31, 28, 31, 30, 31, 30, 31, 31, 30, 31, 30, 31]
# days_of_month_leap_year = [31, 29, 31, 30, 31, 30, 31, 31, 30, 31, 30, 31]


def formatting(date_time: Union[str, datetime], original: Union[str, datetime], target: Union[str, datetime]):
    """
    对传入时间进行指定类型格式化

    :param date_time: 传入的时间，时间常见格式字符串、datetime对象
    :param original: 原始格式，时间常见格式字符串、datetime对象
    :param target: 目标格式，时间常见格式字符串、datetime对象
    :return: result
    """
    if isinstance(original, datetime) and isinstance(target, datetime):
        raise TargetError('original and target cannot be datetime')
    if isinstance(original, datetime) and isinstance(target, str):
        return datetime_to_str(date_time, target)
    if isinstance(original, str) and isinstance(target, datetime):
        return str_to_datetime(date_time, original)
    if isinstance(original, str) and isinstance(target, str):
        return str_to_str(date_time, original, target)
    raise TargetError('input target error')


def datetime_to_str(date_time: datetime, target: str):
    """
    格式化为字符串

    :return: result
    """
    return date_time.strftime(target)


def str_to_datetime(date_time: str, original: str):
    """
    格式化为datetime

    :return: result
    """
    return datetime.strptime(date_time, original)


def str_to_str(date_time: str, original: str, target: str):
    """
    格式化为字符串

    :return: result
    """
    dtime = datetime.strptime(date_time, original)
    return dtime.strftime(target)


def datetime_offset(date_time: datetime, year: int = 0, month: int = 0,
                    day: int = 0, hour: int = 0, minute: int = 0, second: int = 0):
    """
    计算传入时间偏移量

    :param date_time: 需要偏移的时间，datetime
    :param year: offset year
    :param month: offset month
    :param day: offset day
    :param hour: offset hour
    :param minute: offset minute
    :param second: offset second
    :return: result
    """
    if year:
        date_time = date_time + relativedelta(years=year)
    if month:
        date_time = date_time + relativedelta(months=month)
    return date_time + timedelta(days=day, hours=hour, minutes=minute, seconds=second)


def judge_leap_year(year):
    """
    判断是否为闰年
    """
    if year % 400 == 0 or year % 100 == 0 or year % 4 == 0:
        return True
    return False


def clac_datetime_differ(start: datetime, end: datetime, diff_type: str, only: bool = True):
    """
    计算两时间之差

    :param start: 需计算时间1
    :param end: 需计算时间2
    :param diff_type: 需计算类型
    :param only: 是否只计算传入量级，量级向上兼容，非量级计算不严格精准
    :return: result
    """
    # 参数判断
    if not isinstance(start, datetime) or not isinstance(end, datetime):
        raise ParamsError('start or end is not datetime')
    # 允许传参集合
    diff_params = ('year', 'month', 'day', 'hour', 'minute', 'second')
    # 参数判断
    if diff_type not in diff_params:
        raise ParamsError(f'diff_type must in {str(diff_params)}')
    # 相等时间排异
    if start == end:
        return 0
    if start > end:
        start, end = end, start
    return globals()['clac_' + diff_type](start, end, only)


def clac_year(start, end, only):
    if only:
        return int(end.strftime('%Y')) - int(start.strftime('%Y'))
    return round(((end - start).days + 1) / 365, 2)


def clac_month(start, end, only):
    if only:
        return (int(end.strftime('%Y')) - int(start.strftime('%Y'))) * 12 \
               + (int(end.strftime('%m')) - int(start.strftime('%m'))) + 1
    return round(((end - start).days + 1) / 365 * 12, 2)


def clac_day(start, end, only):
    diff = end - start
    if only:
        return diff.days
    return round(diff.days + diff.seconds / 86400, 2)


def clac_hour(start, end, only):
    diff = end - start
    if only:
        return diff.days * 24 + int(diff.seconds / 3600)
    return round(diff.days * 24 + diff.seconds / 3600, 2)


def clac_minute(start, end, only):
    diff = end - start
    if only:
        return diff.days * 1440 + int(diff.seconds / 60)
    return round(diff.days * 1440 + diff.seconds / 60, 2)


def clac_second(start, end, only):
    diff = end - start
    return diff.days * 86400 + diff.seconds



class BaseError(Exception):
    def __init__(self, error_info):
        super().__init__(self)
        self.error_info = error_info

    def __str__(self):
        return self.error_info


class ParamsError(BaseError):
    ...


class TargetError(BaseError):
    pass


class ClientError(BaseError):
    ...


class InitDBError(BaseError):
    ...



import os
import time
import logging


class Log:
    def __init__(self, logger=None, file_dir='./log'):
        """
        指定保存日志的文件路径，日志级别，以及调用文件
        将日志存入到指定的文件中
        """
        # 创建一个logger
        self.logger = logging.getLogger(logger)
        self.logger.setLevel(logging.INFO)
        # 创建一个handler，用于写入日志文件
        self.log_time = time.strftime('%Y_%m_%d')
        if not os.path.exists(file_dir):
            os.mkdir(file_dir)
        self.log_name = os.path.join(file_dir, time.strftime('%Y_%m_%d') + '.log')
        fh = logging.FileHandler(self.log_name, 'a', encoding='utf-8')
        fh.setLevel(logging.INFO)
        # 再创建一个handler，用于输出到控制台
        ch = logging.StreamHandler()
        ch.setLevel(logging.INFO)
        # 定义handler的输出格式
        formatter = logging.Formatter(
            '[%(asctime)s] %(filename)s->%(funcName)s line:%(lineno)d [%(levelname)s]%(message)s')
        fh.setFormatter(formatter)
        ch.setFormatter(formatter)
        # 给logger添加handler
        self.logger.addHandler(fh)
        self.logger.addHandler(ch)
        # 关闭打开的文件
        fh.close()
        ch.close()

    def get_log(self):
        return self.logger



from setuptools import setup, find_packages


VERSION = '1.0.3'
DESCRIPTION = '简化数据库连接、时间处理及log声明'
LONG_DESCRIPTION = '数据库：mongo、mysql、oracle、xugu；时间：格式化、转换、计算；log：声明'

# 配置
setup(
    name='TiamoTools',
    version=VERSION,
    author='wyl',
    author_email='<321081840@qq.com>',
    description=DESCRIPTION,
    long_description=LONG_DESCRIPTION,
    packages=find_packages(),
    python_requires='>=3.8',
    requires=[
        'PyMySQL',
        'pymongo',
        'cx_Oracle'
    ],
    install_requires=[
        'PyMySQL>=1.0.2',
        'pymongo>=3.8.0',
        'cx_Oracle>=8.3.0'
    ],
    keywords=['python', 'super', 'tools'],
    classifiers=[
        'Intended Audience :: Developers',
        'Programming Language :: Python :: 3.8'
    ]
)




from ._pyxgdb import _connect


def Connect(*args, **kwargs):
    # def_charset=""
    if kwargs is None:
        raise TypeError('Connection parameters are incorrect!')
    elif 'host' in kwargs and 'port' in kwargs and 'database' in kwargs and 'user' in kwargs and 'password' in kwargs:
        if kwargs['host'] is None or kwargs['port'] is None or kwargs['database'] is None or kwargs['user'] is None or \
                kwargs['password'] is None:
            raise TypeError('Connection parameters are incorrect!')
    if 'charset' not in kwargs:
        kwargs['charset'] = "utf8"
    if 'usessl' not in kwargs:
        kwargs['usessl'] = "off"
    #    else:
    #        def_charset=kwargs['charset']

    conn = _connect(host=kwargs['host'],
                            port=int(kwargs['port']),
                            database=kwargs['database'],
                            user=kwargs['user'],
                            password=kwargs['password'],
                            charset=kwargs['charset'],
                            usessl=kwargs['usessl'])
    # conn = Pyxgmodule._connect(*args, **kwargs)
    return conn


def reverse(*args, **kwargs):
    _reverse(*args)


connect = Connection = Connect

# version='v2.1.1'

XG_C_NULL = 0
XG_C_BOOL = 1
XG_C_CHAR = 2
XG_C_TINYINT = 3
XG_C_SHORT = 4
XG_C_INTEGER = 5
XG_C_BIGINT = 6
XG_C_FLOAT = 7
XG_C_DOUBLE = 8
XG_C_NUMERIC = 9
XG_C_DATE = 10
XG_C_TIME = 11
XG_C_TIME_TZ = 12
XG_C_DATETIME = 13
XG_C_DATETIME_TZ = 14
XG_C_BINARY = 15
XG_C_NVARBINARY = 18
XG_C_INTERVAL = 21
DATETIME_ASLONG = 23
XG_C_INTERVAL_YEAR_TO_MONTH = 28
XG_C_INTERVAL_DAY_TO_SECOND = 31
XG_C_TIMESTAMP = 13
XG_C_LOB = 40
XG_C_CLOB = 41
XG_C_BLOB = 42
XG_C_REFCUR = 58
XG_C_NCHAR = 62
XG_C_CHARN1 = 63



from collections import Counter
from functools import wraps


class Dict(dict):
    """
    重写字典对象，添加__setitem__，__getitem__魔术方法
    """
    __setattr__ = dict.__setitem__
    __getattr__ = dict.__getitem__

    def __getstate__(self):
        """
        添加可序列化魔术方法，否则协程迟报错（协程任务参数必须可序列化）
        """
        ...


def dict_to_object(data):
    """
    字典转化为Dict对象
    ----------
    data: 无特定类型，dict进行分解，其他赋为对应键的值
    ----------
    Dict对象形字典
    """
    if not isinstance(data, dict):
        return data
    inst = Dict()
    for k, v in data.items():
        inst[k] = dict_to_object(v)
    return inst


def judge_params(template: list, params: dict):
    """
    :param template: 参数模板，连接类内部定义
    :param params: 传入参数
    :return: 对比结果
    """
    if dict(Counter(template)) == dict(Counter(params.keys())):
        return True
    return False


def retry_result(function):
    """
    对增查改方法进行运行结果订正

    :return: result
    """
    @wraps(function)
    def func(*args, **kwargs):
        try:
            res = function(*args, **kwargs)
            if 'search' in function.__name__:
                return res
            return True
        except Exception as e:
            print('%s-%s: %s' % (function.__qualname__, function.__name__, str(e)))
            return False
    return func
